﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace Dayu
{
    class Regulate
    {
        const bool READING = false;
        const bool WRITING = true;
        const int ASCII_SIZE = 1;
        const int BELL = 0x07;
        const int BLOCK_SIZE = 3072;
        const int DOUBLE_KANJI = 4; // SJIS_KANJI_SIZE + SJIS_KANJI_SIZE;
        const int ERROR_CODE = 1;
        const int FILE_ERROR_CODE = -2;
        const int KANA_HANDAKUTEN = 0xDF;
        const int KANA_KUTEN = 0xA1;
        const int KANJI_SIZE = 2;
        const int KATAKANA_SIZE = 1;
        const string DAYU_TEXT = "Dayu.txt";
        const string TEMPORARY_BINARY = "binary.tmp";

        int[] BELL_ASCII_KATAKANA = {
            0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
            0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
            0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F, 
            0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F, 
            0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F, 
            0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x5B, 0x5C, 0x5D, 0x5E, 0x5F, 
            0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 
            0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x7B, 0x7C, 0x7D, 0x7E, 0x07, 
            0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
            0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
            0x07, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xAB, 0xAC, 0xAD, 0xAE, 0xAF, 
            0xB0, 0xB1, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6, 0xB7, 0xB8, 0xB9, 0xBA, 0xBB, 0xBC, 0xBD, 0xBE, 0xBF, 
            0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF, 
            0xD0, 0xD1, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA, 0xDB, 0xDC, 0xDD, 0xDE, 0xDF
        };

        SearchTable s;
        string documentName;

        private int inverted(byte[] c)
        {
            int k = (int)c[0];
            k = k << 8;
            k += c[1];
            return k;
        }

        public int produceBinaryFile(string name, bool no_removing)
        {
            int exit_code = 0;
            int size;

            byte[] buffer = new byte[BLOCK_SIZE];

            try
            {
                File.Delete(TEMPORARY_BINARY);
                Stream ancient = File.OpenRead(name);
                Stream temporary = File.OpenWrite(TEMPORARY_BINARY);

                while ( ( size = ancient.Read(buffer, 0, BLOCK_SIZE) ) > 0 )
                {
                    temporary.Write(buffer, 0, size);
                }

                temporary.Close();
                ancient.Close();

                if (! no_removing)
                {
                    File.Delete(name);
                }

            }
            catch (IOException e252)
            {
                Console.Error.Write("OpenRead() error: ");
                Console.Error.WriteLine(name);
                Error.handler( e252.ToString() );
                exit_code = FILE_ERROR_CODE;
            }
            catch (Exception e253)
            {
                Error.handler( e253.ToString() );
                exit_code = FILE_ERROR_CODE;
            }

            return exit_code;
        }

        public int readingToProduceSJIS_LF(string new_name)
        {
            bool kanji = false;
            byte c;
            byte firstByte = 0x40;
            byte[] a = {0, 0};
            byte[] buffer;
            byte[] w = {0, 0};
            Encoding MS932 = Encoding.GetEncoding(932);
            int size;
            int exit_code = 0;
            string line;

            OpenClose o = new OpenClose(DAYU_TEXT);
            o.openClose(new_name, WRITING);
            new_name = o.newName();

            try
            {
                o.openClose(documentName, READING);
            }
            catch (Exception e254)
            {
                Console.Error.Write("openClose() error: ");
                Console.Error.WriteLine(documentName);
                Error.handler( e254.ToString() );
                return ERROR_CODE;
            }

            SparseSJIS sjis = new SparseSJIS();
            if (! sjis.setting(s))
            {
                Console.Error.Write("readingToProduceSJIS_LF() error: ");
                Console.Error.WriteLine("sjis.setting(s)");
                return ERROR_CODE;
            }

            try
            {
                Stream binaryFile = File.OpenWrite(new_name);
                StreamReader sjisFile = new StreamReader(documentName, MS932);

                for (; ; )
                {
                    line = sjisFile.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    buffer = MS932.GetBytes(line);
                    size = MS932.GetByteCount(line);
                    for (int i = 0; i < size; ++i)
                    {
                        c = buffer[i];

                        if (kanji)
                        {
                            a[0] = firstByte;
                            a[1] = buffer[i];
                            w[0] = sjis.working( inverted(a), 0);
                            w[1] = sjis.working( inverted(a), 1);
                            binaryFile.Write(w, 0, KANJI_SIZE);
                            kanji = false;
                            continue;
                        }

                        if (0x81 <= c && c<= 0xFC)
                        {
                            if (KANA_KUTEN <= c && c <= KANA_HANDAKUTEN)
                            {
                                w[0] = sjis.working(c, 0);
                                w[1] = 0;
                                binaryFile.Write(w, 0, KATAKANA_SIZE);
                                kanji = false;
                            }
                            else
                            {
                                firstByte = c;
                                kanji = true;
                            }

                        }
                        else
                        {
                            w[0] = sjis.working(c, 0);
                            w[1] = 0;
                            binaryFile.Write(w, 0, ASCII_SIZE);
                            kanji = false;
                        }

                    } // for (int i = 0; i < size; ++i)

                    w[0] = 0x0A;
                    w[1] = 0;
                    binaryFile.Write(w, 0, ASCII_SIZE);
                } //for (; ; )

                binaryFile.Close();
                sjisFile.Close();
            }
            catch (IOException e255)
            {
                Error.handler(e255.ToString());
                exit_code = ERROR_CODE;
            }

            return exit_code;
        }

        public int readingToProduceSJIS(string new_name)
        {
            bool kanji = false;
            byte c;
            byte firstByte = 0x40;
            byte[] a = {0, 0};
            byte[] buffer;
            byte[] w = {0, 0};
            Encoding MS932 = Encoding.GetEncoding(932);
            int size;
            int exit_code = 0;
            string line;

            OpenClose o = new OpenClose(DAYU_TEXT);
            o.openClose(new_name, WRITING);
            new_name = o.newName();

            try
            {
                o.openClose(documentName, READING);
            }
            catch (Exception e256)
            {
                Console.Error.Write("openClose() error: ");
                Console.Error.WriteLine(documentName);
                Error.handler( e256.ToString() );
                return ERROR_CODE;
            }

            SparseSJIS sjis = new SparseSJIS();
            if (! sjis.setting(s))
            {
                Console.Error.Write("readingToProduceSJIS() error: ");
                Console.Error.WriteLine("sjis.setting(s)");
                return ERROR_CODE;
            }

            try
            {
                Stream binaryFile = File.OpenWrite(new_name);
                StreamReader sjisFile = new StreamReader(documentName, MS932);

                for (; ; )
                {
                    line = sjisFile.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    buffer = MS932.GetBytes(line);
                    size = MS932.GetByteCount(line);
                    for (int i = 0; i < size; ++i)
                    {
                        c = buffer[i];

                        if (kanji)
                        {
                            a[0] = firstByte;
                            a[1] = buffer[i];
                            w[0] = sjis.working( inverted(a), 0);
                            w[1] = sjis.working( inverted(a), 1);
                            binaryFile.Write(w, 0, KANJI_SIZE);
                            kanji = false;
                            continue;
                        }

                        if (0x81 <= c && c<= 0xFC)
                        {
                            if (KANA_KUTEN <= c && c <= KANA_HANDAKUTEN)
                            {
                                w[0] = sjis.working(c, 0);
                                w[1] = 0;
                                binaryFile.Write(w, 0, KATAKANA_SIZE);
                                kanji = false;
                            }
                            else
                            {
                                firstByte = c;
                                kanji = true;
                            }

                        }
                        else
                        {
                            w[0] = sjis.working(c, 0);
                            w[1] = 0;
                            binaryFile.Write(w, 0, ASCII_SIZE);
                            kanji = false;
                        }

                    } // for (int i = 0; i < size; ++i)

                    w[0] = 0x0D;
                    w[1] = 0x0A;
                    binaryFile.Write(w, 0, 2);
                } //for (; ; )

                binaryFile.Close();
                sjisFile.Close();
            }
            catch (IOException e257)
            {
                Error.handler(e257.ToString());
                exit_code = ERROR_CODE;
            }

            return exit_code;
        }

        public int renameBinaryFile(string name, bool no_removing)
        {
            int exit_code = 0;
            int size;

            byte[] buffer = new byte[BLOCK_SIZE];

            try
            {
                File.Delete(name);
                Stream temporary = File.OpenRead(TEMPORARY_BINARY);
                Stream recent = File.OpenWrite(name);

                while ( ( size = temporary.Read(buffer, 0, BLOCK_SIZE) ) > 0 )
                {
                    recent.Write(buffer, 0, size);
                }

                recent.Close();
                temporary.Close();

                if (! no_removing)
                {
                    File.Delete(TEMPORARY_BINARY);
                }

            }
            catch (IOException e258)
            {
                Error.handler( e258.ToString() );
                exit_code = FILE_ERROR_CODE;
            }
            catch (Exception e259)
            {
                Error.handler( e259.ToString() );
                exit_code = FILE_ERROR_CODE;
            }

            return exit_code;
        }

        public int skippingControlCharactersToProduceSJIS_LF(string new_name)
        {
            bool kanji = false;
            byte c;
            byte firstByte = 0x40;
            byte[] a = {0, 0};
            byte[] buffer;
            byte[] w = {0, 0};
            Encoding MS932 = Encoding.GetEncoding(932);
            int size;
            int exit_code = 0;
            string line;

            OpenClose o = new OpenClose(DAYU_TEXT);
            o.openClose(new_name, WRITING);
            new_name = o.newName();

            try
            {
                o.openClose(documentName, READING);
            }
            catch (Exception e260)
            {
                Console.Error.Write("openClose() error: ");
                Console.Error.WriteLine(documentName);
                Error.handler( e260.ToString() );
                return ERROR_CODE;
            }

            SparseSJIS sjis = new SparseSJIS();
            if (! sjis.setting(s))
            {
                Console.Error.Write("skippingControlCharactersToProduceSJIS_LF() error: ");
                Console.Error.WriteLine("sjis.setting(s)");
                return ERROR_CODE;
            }

            try
            {
                Stream binaryFile = File.OpenWrite(new_name);
                StreamReader sjisFile = new StreamReader(documentName, MS932);

                for (; ; )
                {
                    line = sjisFile.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    buffer = MS932.GetBytes(line);
                    size = MS932.GetByteCount(line);
                    for (int i = 0; i < size; ++i)
                    {
                        c = buffer[i];

                        if (kanji)
                        {
                            a[0] = firstByte;
                            a[1] = buffer[i];
                            w[0] = sjis.working( inverted(a), 0);
                            w[1] = sjis.working( inverted(a), 1);
                            binaryFile.Write(w, 0, KANJI_SIZE);
                            kanji = false;
                            continue;
                        }

                        if (0x81 <= c && c<= 0xFC)
                        {
                            if (KANA_KUTEN <= c && c <= KANA_HANDAKUTEN)
                            {
                                w[0] = sjis.working(c, 0);
                                w[1] = 0;
                                binaryFile.Write(w, 0, KATAKANA_SIZE);
                                kanji = false;
                            }
                            else
                            {
                                firstByte = c;
                                kanji = true;
                            }

                        }
                        else
                        {
                            if (BELL_ASCII_KATAKANA[c] == BELL)
                            {
                                continue; // skipping control characters
                            }
                            else
                            {
                                w[0] = sjis.working(c, 0);
                                w[1] = 0;
                                binaryFile.Write(w, 0, ASCII_SIZE);
                            }

                            kanji = false;
                        }

                    } // for (int i = 0; i < size; ++i)

                    w[0] = 0x0A;
                    w[1] = 0;
                    binaryFile.Write(w, 0, ASCII_SIZE);
                } //for (; ; )

                binaryFile.Close();
                sjisFile.Close();
            }
            catch (IOException e261)
            {
                Error.handler(e261.ToString());
                exit_code = ERROR_CODE;
            }

            return exit_code;
        }

        public int skippingControlCharactersToProduceSJIS(string new_name)
        {
            bool kanji = false;
            byte c;
            byte firstByte = 0x40;
            byte[] a = {0, 0};
            byte[] buffer;
            byte[] w = {0, 0};
            Encoding MS932 = Encoding.GetEncoding(932);
            int size;
            int exit_code = 0;
            string line;

            OpenClose o = new OpenClose(DAYU_TEXT);
            o.openClose(new_name, WRITING);
            new_name = o.newName();

            try
            {
                o.openClose(documentName, READING);
            }
            catch (Exception e262)
            {
                Console.Error.Write("openClose() error: ");
                Console.Error.WriteLine(documentName);
                Error.handler( e262.ToString() );
                return ERROR_CODE;
            }

            SparseSJIS sjis = new SparseSJIS();
            if (! sjis.setting(s))
            {
                Console.Error.Write("skippingControlCharactersToProduceSJIS() error: ");
                Console.Error.WriteLine("sjis.setting(s)");
                return ERROR_CODE;
            }

            try
            {
                Stream binaryFile = File.OpenWrite(new_name);
                StreamReader sjisFile = new StreamReader(documentName, MS932);

                for (; ; )
                {
                    line = sjisFile.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    buffer = MS932.GetBytes(line);
                    size = MS932.GetByteCount(line);
                    for (int i = 0; i < size; ++i)
                    {
                        c = buffer[i];

                        if (kanji)
                        {
                            a[0] = firstByte;
                            a[1] = buffer[i];
                            w[0] = sjis.working( inverted(a), 0);
                            w[1] = sjis.working( inverted(a), 1);
                            binaryFile.Write(w, 0, KANJI_SIZE);
                            kanji = false;
                            continue;
                        }

                        if (0x81 <= c && c<= 0xFC)
                        {
                            if (KANA_KUTEN <= c && c <= KANA_HANDAKUTEN)
                            {
                                w[0] = sjis.working(c, 0);
                                w[1] = 0;
                                binaryFile.Write(w, 0, KATAKANA_SIZE);
                                kanji = false;
                            }
                            else
                            {
                                firstByte = c;
                                kanji = true;
                            }

                        }
                        else
                        {
                            if (BELL_ASCII_KATAKANA[c] == BELL)
                            {
                                continue; // skipping control characters
                            }
                            else
                            {
                                w[0] = sjis.working(c, 0);
                                w[1] = 0;
                                binaryFile.Write(w, 0, ASCII_SIZE);
                            }

                            kanji = false;
                        }

                    } // for (int i = 0; i < size; ++i)

                    w[0] = 0x0D;
                    w[1] = 0x0A;
                    binaryFile.Write(w, 0, 2);
                } //for (; ; )

                binaryFile.Close();
                sjisFile.Close();
            }
            catch (IOException e263)
            {
                Error.handler(e263.ToString());
                exit_code = ERROR_CODE;
            }

            return exit_code;
        }

        public Regulate(SearchTable search_table, string d)
        {
            s = search_table;
            documentName = d;
        }

    }
}
