﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace Hetn
{
    class AscendingOrder
    {
        const bool NO_APPENDING = false;
        const char COMMA = ',';
        const char JOINT = ' '; // Sortk uses ' ' as a joint.
        const char QUOTE = '"';
        const int ERROR_CODE = 1;
        const int NOT_CONVERTED = -1;
        const string DETAILS_BEGINNING = "</string><hetn>";
        const string DOUBLE_COUNT_BEGINNING = "<double_count>";
        const string DOUBLE_COUNT_END = "</double_count><double>";
        const string DOUBLE_VALUE_END = "</double><double>";
        const string FINAL_DOUBLE_VALUE_END = "</double><hetn>";
        const string FINAL_INTEGER_VALUE_END = "</integer><hetn>";
        const string INTEGER_COUNT_BEGINNING = "<integer_count>";
        const string INTEGER_COUNT_END = "</integer_count><integer>";
        const string INTEGER_VALUE_END = "</integer><integer>";
        const string N_0TO9_COMMA_MINUS_QUOTE = "0123456789,-\"";
        const string N_0TO9_DOT_MINUS_QUOTE = "0123456789.-\"";
        const string SUMMARY_SIZE_BEGINNING = "<string_length>";
        const string SUMMARY_SIZE_END = "</string_length><string>";
        const string UNIQUE_ID_BEGINNING = "<summary_details><unique_id>";
        const string UNIQUE_ID_END = "</unique_id>";

        bool add_ID, carriage_return, report_ID;
        char joint;
        Encoding encoded;
        Encoding UTF_8 = Encoding.UTF8;
        int line_count;
        int DETAILS_BEGINNING_SIZE, DOUBLE_COUNT_BEGINNING_SIZE, DOUBLE_COUNT_END_SIZE, DOUBLE_VALUE_END_SIZE, FINAL_DOUBLE_VALUE_END_SIZE, FINAL_INTEGER_VALUE_END_SIZE, INTEGER_COUNT_BEGINNING_SIZE, INTEGER_COUNT_END_SIZE, INTEGER_VALUE_END_SIZE, SUMMARY_SIZE_BEGINNING_SIZE, SUMMARY_SIZE_END_SIZE, UNIQUE_ID_BEGINNING_SIZE, UNIQUE_ID_END_SIZE;
        string new_name, xml_name;

        private double sequence_to_double(string phrase)
        {
            char c;
            double a = NOT_CONVERTED;
            int q = -1;
            StringBuilder os = new StringBuilder();

            for (int i = 0; i < phrase.Length; ++i)
            {
                c = phrase[i];
                q = N_0TO9_DOT_MINUS_QUOTE.IndexOf(c);
                if (q >= 0)
                {
                    switch (c)
                    {
                        case QUOTE:
                            continue; // Removing QUOTE from phrase
                        default:
                            os.Append(c);
                            break;
                    }

                }
                else
                {
                    break;
                }

            }

            phrase = os.ToString();

            try
            {
                a = Convert.ToDouble(phrase);
            }
            catch (FormatException e474)
            {
                Error.handler(e474.ToString());
            }

            return a;
        }

        private int sequence_to_integer(string phrase)
        {
            char c;
            int a = NOT_CONVERTED;
            int q = -1;
            StringBuilder os = new StringBuilder();

            for (int i = 0; i < phrase.Length; ++i)
            {
                c = phrase[i];
                q = N_0TO9_COMMA_MINUS_QUOTE.IndexOf(c);
                if (q >= 0)
                {
                    switch (c)
                    {
                        case COMMA:
                            continue; // Removing COMMA from phrase
                        case QUOTE:
                            continue; // Removing QUOTE from phrase
                        default:
                            os.Append(c);
                            break;
                    }

                }
                else
                {
                    break;
                }

            }

            phrase = os.ToString();

            try
            {
                a = Convert.ToInt32(phrase);
            }
            catch (FormatException e471)
            {
                Error.handler(e471.ToString());
            }

            return a;
        }

        private string postediting(string s)
        {
            char c;
            StringBuilder os = new StringBuilder();
            for (int i = 0; i < s.Length; ++i)
            {
                c = s[i];
                switch (c)
                {
                    case ' ':
                        os.Append('!'); // It is necessary to replace ' ' with an exclamation mark so that Sortk can use ' ' as a joint.
                        break;
                    case '!':
                        os.Append(JOINT); // Sortk uses ' ' as a joint. Please note that if the text includes an exclamation mark, Sortk confuses it with a joint.
                        break;
                    default:
                        if (c == joint)
                        {
                            os.Append(JOINT); // Sortk uses ' ' as a joint.
                        }
                        else
                        {
                            os.Append(c);
                        }

                        break;
                }

            }

            return os.ToString();
        }

        public AscendingOrder(Encoding e, string n, string x, int c, char j, bool a, bool i, bool r)
        {
            encoded = e;
            new_name = n;
            xml_name = x;
            line_count = c;
            joint  = j;
            add_ID = a;
            carriage_return = i;
            report_ID = r;

            DETAILS_BEGINNING_SIZE = DETAILS_BEGINNING.Length;
            DOUBLE_COUNT_BEGINNING_SIZE = DOUBLE_COUNT_BEGINNING.Length;
            DOUBLE_COUNT_END_SIZE = DOUBLE_COUNT_END.Length;
            DOUBLE_VALUE_END_SIZE = DOUBLE_VALUE_END.Length;
            FINAL_DOUBLE_VALUE_END_SIZE = FINAL_DOUBLE_VALUE_END.Length;
            FINAL_INTEGER_VALUE_END_SIZE = FINAL_INTEGER_VALUE_END.Length;
            INTEGER_COUNT_BEGINNING_SIZE = INTEGER_COUNT_BEGINNING.Length;
            INTEGER_COUNT_END_SIZE = INTEGER_COUNT_END.Length;
            INTEGER_VALUE_END_SIZE = INTEGER_VALUE_END.Length;
            SUMMARY_SIZE_BEGINNING_SIZE = SUMMARY_SIZE_BEGINNING.Length;
            SUMMARY_SIZE_END_SIZE = SUMMARY_SIZE_END.Length;
            UNIQUE_ID_BEGINNING_SIZE = UNIQUE_ID_BEGINNING.Length;
            UNIQUE_ID_END_SIZE = UNIQUE_ID_END.Length;
        }

        public int arranging_to_produce_text()
        {
            Block b;
            CheckID c = new CheckID();
            int beginning, from, end, q, summary_size, new_name_size;
            int exit_code = 0;
            string details, line, summary, unique_id, value;
            List<Block> blockArray = new List<Block>();

            try
            {
                StreamReader file = new StreamReader(xml_name, UTF_8);

                if (add_ID)
                {

                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        end = line.IndexOf(SUMMARY_SIZE_END);
                        if (end >= 0)
                        {
                            from = end + SUMMARY_SIZE_END_SIZE;
                            beginning = line.IndexOf(SUMMARY_SIZE_BEGINNING);
                            if (beginning >= 0)
                            {
                                beginning += SUMMARY_SIZE_BEGINNING_SIZE;
                                value = line.Substring(beginning, end - beginning );
                                summary_size = sequence_to_integer(value);
                                summary = postediting( line.Substring(from, summary_size) );

                                q = line.IndexOf(DETAILS_BEGINNING);
                                q += DETAILS_BEGINNING_SIZE;
                                details = line.Substring(q);

                                end = line.IndexOf(UNIQUE_ID_END);
                                if (end >= 0)
                                {
                                    beginning = line.IndexOf(UNIQUE_ID_BEGINNING);
                                    if (beginning >= 0)
                                    {
                                        beginning += UNIQUE_ID_BEGINNING_SIZE;
                                        unique_id = line.Substring(beginning, end - beginning);

                                        b = new Block(c, details, summary, unique_id);
                                        blockArray.Add(b);
                                        if (blockArray.Count > line_count)
                                        {
                                            break;
                                        }

                                    }

                                } // if (end >= 0)

                            } // if (beginning >= 0)

                        } // if (end >= 0)

                    } // for (; ; )

                }
                else
                {
                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        end = line.IndexOf(SUMMARY_SIZE_END);
                        if (end >= 0)
                        {
                            from = end + SUMMARY_SIZE_END_SIZE;
                            beginning = line.IndexOf(SUMMARY_SIZE_BEGINNING);
                            if (beginning >= 0)
                            {
                                beginning += SUMMARY_SIZE_BEGINNING_SIZE;
                                value = line.Substring(beginning, end - beginning );
                                summary_size = sequence_to_integer(value);
                                summary = postediting( line.Substring(from, summary_size) );

                                q = line.IndexOf(DETAILS_BEGINNING);
                                q += DETAILS_BEGINNING_SIZE;
                                details = line.Substring(q);

                                b = new Block(details, summary);
                                blockArray.Add(b);
                                if (blockArray.Count > line_count)
                                {
                                    break;
                                }

                            }

                        }

                    }

                }

                file.Close();
            }
            catch (IOException e458)
            {
                Error.handler( e458.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(xml_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e461)
            {
                Error.handler( e461.ToString() );
                exit_code = ERROR_CODE;
            }

            new_name_size = new_name.Length;
            if (new_name_size == 0)
            {
                blockArray.Sort();
                blockArray.ForEach(g => Console.WriteLine(g.ToString()));
            }
            else
            {
                try
                {
                    if (carriage_return) 
                    {
                        StreamWriter newFile = new StreamWriter(new_name, NO_APPENDING, encoded);
                        blockArray.Sort();
                        blockArray.ForEach(g => newFile.WriteLine(g.ToString()));
                        newFile.Close();
                    }
                    else
                    {
                        ComboAppend ca = new ComboAppend('\n');
                        blockArray.Sort();
                        blockArray.ForEach(g => ca.separatorFollows(g.ToString()));

                        File.Delete(new_name);
                        Stream newFile = File.OpenWrite(new_name);
                        byte[] combo = encoded.GetBytes( ca.built() );
                        newFile.Write(combo, 0, combo.Length);
                        newFile.Close();
                    }

                }
                catch (IOException e462)
                {
                    Error.handler( e462.ToString() );
                    Console.Error.Write("StreamWriter() error: ");
                    Console.Error.WriteLine(new_name);
                    exit_code = ERROR_CODE;
                }
                catch (Exception e465)
                {
                    Error.handler( e465.ToString() );
                    exit_code = ERROR_CODE;
                }

            }

            if (report_ID)
            {
                c.reportID();
            }

            return exit_code;
        }

        public int arranging_doubles_to_produce_text()
        {
            CheckID c = new CheckID();
            int beginning, double_count, double_count_1, from, end, line_ID, q, to, new_name_size;
            int exit_code = 0;
            double[] s;
            DoubleBlock b;
            string details, line, value;
            List<DoubleBlock> blockArray = new List<DoubleBlock>();

            try
            {
                StreamReader file = new StreamReader(xml_name, UTF_8);

                if (add_ID)
                {
                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        end = line.IndexOf(DOUBLE_COUNT_END);
                        if (end >= 0)
                        {
                            from = end + DOUBLE_COUNT_END_SIZE;
                            beginning = line.IndexOf(DOUBLE_COUNT_BEGINNING);
                            if (beginning >= 0)
                            {
                                beginning += DOUBLE_COUNT_BEGINNING_SIZE;
                                value = line.Substring(beginning, end - beginning );
                                double_count = sequence_to_integer(value);
                                s = new double[double_count];

                                double_count_1 = double_count - 1;
                                if (double_count_1 > 0)
                                {
                                    to = line.IndexOf(DOUBLE_VALUE_END);

                                    for (int k = 0; k < double_count_1; ++k)
                                    {
                                        value = line.Substring(from, to - from );
                                        s[k] = sequence_to_double(value);
                                        from = to + DOUBLE_VALUE_END_SIZE;
                                        to = line.IndexOf(DOUBLE_VALUE_END, from);
                                    }

                                    to = line.IndexOf(FINAL_DOUBLE_VALUE_END, from);
                                    value = line.Substring(from, to - from );
                                    s[double_count_1] = sequence_to_double(value);
                                }
                                else
                                {
                                    to = line.IndexOf(FINAL_DOUBLE_VALUE_END, from);
                                    value = line.Substring(from, to - from );
                                    s[0] = sequence_to_double(value);
                                }

                                q = to + FINAL_DOUBLE_VALUE_END_SIZE;
                                details = line.Substring(q);

                                end = line.IndexOf(UNIQUE_ID_END);
                                if (end >= 0)
                                {
                                    beginning = line.IndexOf(UNIQUE_ID_BEGINNING);
                                    if (beginning >= 0)
                                    {
                                        beginning += UNIQUE_ID_BEGINNING_SIZE;
                                        value = line.Substring(beginning, end - beginning);
                                        line_ID = sequence_to_integer(value);

                                        b = new DoubleBlock(c, details, double_count, line_ID);
                                        b.setting(s);
                                        blockArray.Add(b);
                                        if (blockArray.Count > line_count)
                                        {
                                            break;
                                        }

                                    }

                                } // if (end >= 0)

                            } // if (beginning >= 0)

                        } // if (end >= 0)

                    } // for (; ; )

                }
                else
                {
                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        end = line.IndexOf(DOUBLE_COUNT_END);
                        if (end >= 0)
                        {
                            from = end + DOUBLE_COUNT_END_SIZE;
                            beginning = line.IndexOf(DOUBLE_COUNT_BEGINNING);
                            if (beginning >= 0)
                            {
                                beginning += DOUBLE_COUNT_BEGINNING_SIZE;
                                value = line.Substring(beginning, end - beginning );
                                double_count = sequence_to_integer(value);
                                s = new double[double_count];

                                double_count_1 = double_count - 1;
                                if (double_count_1 > 0)
                                {
                                    to = line.IndexOf(DOUBLE_VALUE_END);

                                    for (int k = 0; k < double_count_1; ++k)
                                    {
                                        value = line.Substring(from, to - from );
                                        s[k] = sequence_to_double(value);
                                        from = to + DOUBLE_VALUE_END_SIZE;
                                        to = line.IndexOf(DOUBLE_VALUE_END, from);
                                    }

                                    to = line.IndexOf(FINAL_DOUBLE_VALUE_END, from);
                                    value = line.Substring(from, to - from );
                                    s[double_count_1] = sequence_to_double(value);
                                }
                                else
                                {
                                    to = line.IndexOf(FINAL_DOUBLE_VALUE_END, from);
                                    value = line.Substring(from, to - from );
                                    s[0] = sequence_to_double(value);
                                }

                                q = to + FINAL_DOUBLE_VALUE_END_SIZE;
                                details = line.Substring(q);

                                b = new DoubleBlock(details, double_count);
                                b.setting(s);
                                blockArray.Add(b);
                                if (blockArray.Count > line_count)
                                {
                                    break;
                                }

                            }

                        }

                    }

                }

                file.Close();
            }
            catch (IOException e309)
            {
                Error.handler( e309.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(xml_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e310)
            {
                Error.handler( e310.ToString() );
                exit_code = ERROR_CODE;
            }

            new_name_size = new_name.Length;
            if (new_name_size == 0)
            {
                blockArray.Sort();
                blockArray.ForEach(g => Console.WriteLine(g.ToString()));
            }
            else
            {
                try
                {
                    if (carriage_return) 
                    {
                        StreamWriter newFile = new StreamWriter(new_name, NO_APPENDING, encoded);
                        blockArray.Sort();
                        blockArray.ForEach(g => newFile.WriteLine(g.ToString()));
                        newFile.Close();
                    }
                    else
                    {
                        ComboAppend ca = new ComboAppend('\n');
                        blockArray.Sort();
                        blockArray.ForEach(g => ca.separatorFollows(g.ToString()));

                        File.Delete(new_name);
                        Stream newFile = File.OpenWrite(new_name);
                        byte[] combo = encoded.GetBytes( ca.built() );
                        newFile.Write(combo, 0, combo.Length);
                        newFile.Close();
                    }

                }
                catch (IOException e311)
                {
                    Error.handler( e311.ToString() );
                    Console.Error.Write("StreamWriter() error: ");
                    Console.Error.WriteLine(new_name);
                    exit_code = ERROR_CODE;
                }
                catch (Exception e312)
                {
                    Error.handler( e312.ToString() );
                    exit_code = ERROR_CODE;
                }

            }

            if (report_ID)
            {
                c.reportID();
            }

            return exit_code;
        }

        public int arranging_integers_to_produce_text()
        {
            CheckID c = new CheckID();
            int beginning, from, end, integer_count, integer_count_1, line_ID, q, to, new_name_size;
            int exit_code = 0;
            int[] s;
            IntegerBlock b;
            string details, line, value;
            List<IntegerBlock> blockArray = new List<IntegerBlock>();

            try
            {
                StreamReader file = new StreamReader(xml_name, UTF_8);

                if (add_ID)
                {
                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        end = line.IndexOf(INTEGER_COUNT_END);
                        if (end >= 0)
                        {
                            from = end + INTEGER_COUNT_END_SIZE;
                            beginning = line.IndexOf(INTEGER_COUNT_BEGINNING);
                            if (beginning >= 0)
                            {
                                beginning += INTEGER_COUNT_BEGINNING_SIZE;
                                value = line.Substring(beginning, end - beginning );
                                integer_count = sequence_to_integer(value);
                                s = new int[integer_count];

                                integer_count_1 = integer_count - 1;
                                if (integer_count_1 > 0)
                                {
                                    to = line.IndexOf(INTEGER_VALUE_END);

                                    for (int k = 0; k < integer_count_1; ++k)
                                    {
                                        value = line.Substring(from, to - from );
                                        s[k] = sequence_to_integer(value);
                                        from = to + INTEGER_VALUE_END_SIZE;
                                        to = line.IndexOf(INTEGER_VALUE_END, from);
                                    }

                                    to = line.IndexOf(FINAL_INTEGER_VALUE_END, from);
                                    value = line.Substring(from, to - from );
                                    s[integer_count_1] = sequence_to_integer(value);
                                }
                                else
                                {
                                    to = line.IndexOf(FINAL_INTEGER_VALUE_END, from);
                                    value = line.Substring(from, to - from );
                                    s[0] = sequence_to_integer(value);
                                }

                                q = to + FINAL_INTEGER_VALUE_END_SIZE;
                                details = line.Substring(q);

                                end = line.IndexOf(UNIQUE_ID_END);
                                if (end >= 0)
                                {
                                    beginning = line.IndexOf(UNIQUE_ID_BEGINNING);
                                    if (beginning >= 0)
                                    {
                                        beginning += UNIQUE_ID_BEGINNING_SIZE;
                                        value = line.Substring(beginning, end - beginning);
                                        line_ID = sequence_to_integer(value);

                                        b = new IntegerBlock(c, details, integer_count, line_ID);
                                        b.setting(s);
                                        blockArray.Add(b);
                                        if (blockArray.Count > line_count)
                                        {
                                            break;
                                        }

                                    }

                                } // if (end >= 0)

                            } // if (beginning >= 0)

                        } // if (end >= 0)

                    } // for (; ; )

                }
                else
                {
                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        end = line.IndexOf(INTEGER_COUNT_END);
                        if (end >= 0)
                        {
                            from = end + INTEGER_COUNT_END_SIZE;
                            beginning = line.IndexOf(INTEGER_COUNT_BEGINNING);
                            if (beginning >= 0)
                            {
                                beginning += INTEGER_COUNT_BEGINNING_SIZE;
                                value = line.Substring(beginning, end - beginning );
                                integer_count = sequence_to_integer(value);
                                s = new int[integer_count];

                                integer_count_1 = integer_count - 1;
                                if (integer_count_1 > 0)
                                {
                                    to = line.IndexOf(INTEGER_VALUE_END);

                                    for (int k = 0; k < integer_count_1; ++k)
                                    {
                                        value = line.Substring(from, to - from );
                                        s[k] = sequence_to_integer(value);
                                        from = to + INTEGER_VALUE_END_SIZE;
                                        to = line.IndexOf(INTEGER_VALUE_END, from);
                                    }

                                    to = line.IndexOf(FINAL_INTEGER_VALUE_END, from);
                                    value = line.Substring(from, to - from );
                                    s[integer_count_1] = sequence_to_integer(value);
                                }
                                else
                                {
                                    to = line.IndexOf(FINAL_INTEGER_VALUE_END, from);
                                    value = line.Substring(from, to - from );
                                    s[0] = sequence_to_integer(value);
                                }

                                q = to + FINAL_INTEGER_VALUE_END_SIZE;
                                details = line.Substring(q);

                                b = new IntegerBlock(details, integer_count);
                                b.setting(s);
                                blockArray.Add(b);
                                if (blockArray.Count > line_count)
                                {
                                    break;
                                }

                            }

                        }

                    }

                }

                file.Close();
            }
            catch (IOException e299)
            {
                Error.handler( e299.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(xml_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e300)
            {
                Error.handler( e300.ToString() );
                exit_code = ERROR_CODE;
            }

            new_name_size = new_name.Length;
            if (new_name_size == 0)
            {
                blockArray.Sort();
                blockArray.ForEach(g => Console.WriteLine(g.ToString()));
            }
            else
            {
                try
                {
                    if (carriage_return) 
                    {
                        StreamWriter newFile = new StreamWriter(new_name, NO_APPENDING, encoded);
                        blockArray.Sort();
                        blockArray.ForEach(g => newFile.WriteLine(g.ToString()));
                        newFile.Close();
                    }
                    else
                    {
                        ComboAppend ca = new ComboAppend('\n');
                        blockArray.Sort();
                        blockArray.ForEach(g => ca.separatorFollows(g.ToString()));

                        File.Delete(new_name);
                        Stream newFile = File.OpenWrite(new_name);
                        byte[] combo = encoded.GetBytes( ca.built() );
                        newFile.Write(combo, 0, combo.Length);
                        newFile.Close();
                    }

                }
                catch (IOException e301)
                {
                    Error.handler( e301.ToString() );
                    Console.Error.Write("StreamWriter() error: ");
                    Console.Error.WriteLine(new_name);
                    exit_code = ERROR_CODE;
                }
                catch (Exception e302)
                {
                    Error.handler( e302.ToString() );
                    exit_code = ERROR_CODE;
                }

            }

            if (report_ID)
            {
                c.reportID();
            }

            return exit_code;
        }

        public int case_insensitive_arranging_to_produce_text()
        {
            UpperCaseBlock b;
            CheckID c = new CheckID();
            int beginning, from, end, q, summary_size, new_name_size;
            int exit_code = 0;
            string details, line, summary, unique_id, value;
            List<UpperCaseBlock> blockArray = new List<UpperCaseBlock>();

            try
            {
                StreamReader file = new StreamReader(xml_name, UTF_8);

                if (add_ID)
                {

                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        end = line.IndexOf(SUMMARY_SIZE_END);
                        if (end >= 0)
                        {
                            from = end + SUMMARY_SIZE_END_SIZE;
                            beginning = line.IndexOf(SUMMARY_SIZE_BEGINNING);
                            if (beginning >= 0)
                            {
                                beginning += SUMMARY_SIZE_BEGINNING_SIZE;
                                value = line.Substring(beginning, end - beginning );
                                summary_size = sequence_to_integer(value);
                                summary = postediting( line.Substring(from, summary_size) );

                                q = line.IndexOf(DETAILS_BEGINNING);
                                q += DETAILS_BEGINNING_SIZE;
                                details = line.Substring(q);

                                end = line.IndexOf(UNIQUE_ID_END);
                                if (end >= 0)
                                {
                                    beginning = line.IndexOf(UNIQUE_ID_BEGINNING);
                                    if (beginning >= 0)
                                    {
                                        beginning += UNIQUE_ID_BEGINNING_SIZE;
                                        unique_id = line.Substring(beginning, end - beginning);

                                        b = new UpperCaseBlock(c, details, summary, unique_id);
                                        blockArray.Add(b);
                                        if (blockArray.Count > line_count)
                                        {
                                            break;
                                        }

                                    }

                                } // if (end >= 0)

                            } // if (beginning >= 0)

                        } // if (end >= 0)

                    } // for (; ; )

                }
                else
                {
                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        end = line.IndexOf(SUMMARY_SIZE_END);
                        if (end >= 0)
                        {
                            from = end + SUMMARY_SIZE_END_SIZE;
                            beginning = line.IndexOf(SUMMARY_SIZE_BEGINNING);
                            if (beginning >= 0)
                            {
                                beginning += SUMMARY_SIZE_BEGINNING_SIZE;
                                value = line.Substring(beginning, end - beginning );
                                summary_size = sequence_to_integer(value);
                                summary = postediting( line.Substring(from, summary_size) );

                                q = line.IndexOf(DETAILS_BEGINNING);
                                q += DETAILS_BEGINNING_SIZE;
                                details = line.Substring(q);

                                b = new UpperCaseBlock(details, summary);
                                blockArray.Add(b);
                                if (blockArray.Count > line_count)
                                {
                                    break;
                                }

                            }

                        }

                    }

                }

                file.Close();
            }
            catch (IOException e361)
            {
                Error.handler( e361.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(xml_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e362)
            {
                Error.handler( e362.ToString() );
                exit_code = ERROR_CODE;
            }

            new_name_size = new_name.Length;
            if (new_name_size == 0)
            {
                blockArray.Sort();
                blockArray.ForEach(g => Console.WriteLine(g.ToString()));
            }
            else
            {
                try
                {
                    if (carriage_return) 
                    {
                        StreamWriter newFile = new StreamWriter(new_name, NO_APPENDING, encoded);
                        blockArray.Sort();
                        blockArray.ForEach(g => newFile.WriteLine(g.ToString()));
                        newFile.Close();
                    }
                    else
                    {
                        ComboAppend ca = new ComboAppend('\n');
                        blockArray.Sort();
                        blockArray.ForEach(g => ca.separatorFollows(g.ToString()));

                        File.Delete(new_name);
                        Stream newFile = File.OpenWrite(new_name);
                        byte[] combo = encoded.GetBytes( ca.built() );
                        newFile.Write(combo, 0, combo.Length);
                        newFile.Close();
                    }

                }
                catch (IOException e363)
                {
                    Error.handler( e363.ToString() );
                    Console.Error.Write("StreamWriter() error: ");
                    Console.Error.WriteLine(new_name);
                    exit_code = ERROR_CODE;
                }
                catch (Exception e364)
                {
                    Error.handler( e364.ToString() );
                    exit_code = ERROR_CODE;
                }

            }

            if (report_ID)
            {
                c.reportID();
            }

            return exit_code;
        }

    }
}
