﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Xml;

namespace Sortk
{
    class StandardInput
    {
        const bool LISTED = true;
        const bool NO_APPENDING = false;
        const char CLEAR = '0';
        const char JOINT = ' '; // Sortk uses ' ' as a joint.
        const char GAP = ' ';
        const int ERROR_CODE = -1;
        const string EMPTY = "";
        const string SUTAI_XML = "sutai.xml";
        const string XML_VERSION_ENCODING = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>";
        const string XML_STYLESHEET_TYPE = "<?xml-stylesheet type=\"text/xsl\" href=\"sutai.xsl\" ?>";

        bool full_house, remove_separators;
        char joint, quote, separator;
        Encoding encoded;
        Encoding UTF_8 = Encoding.UTF8;
        int field_count, upper_limit;
        string pattern;
        XmlDocument x;
        XmlElement record, sutai;

        private string postediting(string s)
        {
            char c;
            StringBuilder os = new StringBuilder();
            for (int i = 0; i < s.Length; ++i)
            {
                c = s[i];
                switch (c)
                {
                    case ' ':
                        os.Append('!'); // It is necessary to replace ' ' with an exclamation mark so that Sortk can use ' ' as a joint.
                        break;
                    case '!':
                        os.Append(JOINT); // Sortk uses ' ' as a joint. Please note that if the text includes an exclamation mark, Sortk confuses it with a joint.
                        break;
                    default:
                        if (c == joint)
                        {
                            os.Append(JOINT); // Sortk uses ' ' as a joint.
                        }
                        else
                        {
                            os.Append(c);
                        }

                        break;
                }

            }

            return os.ToString();
        }

        public StandardInput(Encoding e, string p, char j, char qu, char s, int c, int u, bool v)
        {
            encoded = e;
            pattern = p;
            joint = j;
            quote = qu;
            separator = s;
            field_count = c;
            upper_limit = u;
            remove_separators = v;

            int q = pattern.IndexOf(CLEAR);
            full_house = (q >= 0) ? false : true;

            x = new XmlDocument();
            sutai = x.CreateElement("sutai_xml");
            x.AppendChild(sutai);
        }

        public int counting()
        {
            byte few_fields;
            int f, shared_field_count;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    line = Console.ReadLine();
                    if (line == null)
                    {
                        return ERROR_CODE;
                    }

                    line_count += 1;

                    d = new SegmentedSummary(separator, few_fields, remove_separators);
                    d.setting(joint, quote);
                    d.segmented_summary(line);
                    shared_field_count = d.fieldCount();

                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        d = new SegmentedSummary(separator, few_fields, remove_separators);
                        d.setting(joint, quote);
                        d.segmented_summary(line);
                        f = d.fieldCount();
                        if (f != shared_field_count)
                        {
                            Console.WriteLine(line);
                        }

                    }

                }
                else
                {
                    line = Console.ReadLine();
                    if (line == null)
                    {
                        return ERROR_CODE;
                    }

                    line_count += 1;

                    d = new SegmentedSummary(pattern, field_count, separator, remove_separators);
                    d.setting(joint, quote);
                    d.segmented_summary(line);
                    shared_field_count = d.fieldCount();

                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        d = new SegmentedSummary(pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.segmented_summary(line);
                        f = d.fieldCount();
                        if (f != shared_field_count)
                        {
                            Console.WriteLine(line);
                        }

                    }

                }

            }
            catch (IOException e383)
            {
                Error.handler( e383.ToString() );
                line_count = ERROR_CODE;
            }
            catch (Exception e384)
            {
                Error.handler( e384.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int line_for_each_summary()
        {
            byte few_fields;
            HashSet<string> hashSet = new HashSet<string>();
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            if (d.uniqueCombo(hashSet))
                            {
                                record = d.phraseElement(x, line);
                                sutai.AppendChild(record);
                            }

                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            if ( d.uniqueCombo(hashSet) )
                            {
                                record = d.phraseElement(x, line);
                                sutai.AppendChild(record);
                            }

                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        d = new SegmentedSummary(pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.segmented_summary(line);
                        if ( d.uniqueCombo(hashSet) )
                        {
                            record = d.phraseElement(x, line);
                            sutai.AppendChild(record);
                        }

                    }

                    newFile.Write( x.OuterXml );
                }

                newFile.Close();
            }
            catch (IOException e385)
            {
                Error.handler( e385.ToString() );
                line_count = ERROR_CODE;
            }
            catch (Exception e386)
            {
                Error.handler( e386.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int line_for_each_double_summary()
        {
            byte few_fields;
            HashSet<string> hashSet = new HashSet<string>();
            FieldsList f;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_double_from_field(line);
                            if ( d.uniqueFieldsList(hashSet) )
                            {
                                record = d.doubleElement(x, line);
                                sutai.AppendChild(record);
                            }

                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_double_from_field(line);
                            if ( d.uniqueFieldsList(hashSet) )
                            {
                                record = d.doubleElement(x, line);
                                sutai.AppendChild(record);
                            }

                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        f = new FieldsList();
                        d = new SegmentedSummary(f, pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.extracting_double_from_field(line);
                        if ( d.uniqueFieldsList(hashSet) )
                        {
                            record = d.doubleElement(x, line);
                            sutai.AppendChild(record);
                        }

                    }

                    newFile.Write( x.OuterXml );
                }

                newFile.Close();
            }
            catch (IOException e387)
            {
                Error.handler( e387.ToString() );
                line_count = ERROR_CODE;
            }
            catch (Exception e388)
            {
                Error.handler( e388.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int line_for_each_integer_summary()
        {
            byte few_fields;
            HashSet<string> hashSet = new HashSet<string>();
            FieldsList f;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_integer_from_field(line);
                            if ( d.uniqueFieldsList(hashSet) )
                            {
                                record = d.integerElement(x, line);
                                sutai.AppendChild(record);
                            }

                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_integer_from_field(line);
                            if ( d.uniqueFieldsList(hashSet) )
                            {
                                record = d.integerElement(x, line);
                                sutai.AppendChild(record);
                            }

                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        f = new FieldsList();
                        d = new SegmentedSummary(f, pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.extracting_integer_from_field(line);
                        if ( d.uniqueFieldsList(hashSet) )
                        {
                            record = d.integerElement(x, line);
                            sutai.AppendChild(record);
                        }

                    }

                    newFile.Write( x.OuterXml );
                }

                newFile.Close();
            }
            catch (IOException e389)
            {
                Error.handler( e389.ToString() );
                line_count = ERROR_CODE;
            }
            catch (Exception e390)
            {
                Error.handler( e390.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int selecting()
        {
            byte few_fields;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            record = d.phraseElement(x, line);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            record = d.phraseElement(x, line);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        d = new SegmentedSummary(pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.segmented_summary(line);
                        record = d.phraseElement(x, line);
                        sutai.AppendChild(record);
                    }

                    newFile.Write( x.OuterXml );
                }

                newFile.Close();
            }
            catch (IOException e391)
            {
                Error.handler( e391.ToString() );
                line_count = ERROR_CODE;
            }
            catch (Exception e392)
            {
                Error.handler( e392.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int selecting_doubles()
        {
            byte few_fields;
            FieldsList f;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_double_from_field(line);
                            record = d.doubleElement(x, line);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_double_from_field(line);
                            record = d.doubleElement(x, line);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        f = new FieldsList();
                        d = new SegmentedSummary(f, pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.extracting_double_from_field(line);
                        record = d.doubleElement(x, line);
                        sutai.AppendChild(record);
                    }

                    newFile.Write( x.OuterXml );
                }

                newFile.Close();
            }
            catch (IOException e393)
            {
                Error.handler( e393.ToString() );
                line_count = ERROR_CODE;
            }
            catch (Exception e394)
            {
                Error.handler( e394.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int selecting_integers()
        {
            byte few_fields;
            FieldsList f;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_integer_from_field(line);
                            record = d.integerElement(x, line);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_integer_from_field(line);
                            record = d.integerElement(x, line);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        f = new FieldsList();
                        d = new SegmentedSummary(f, pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.extracting_integer_from_field(line);
                        record = d.integerElement(x, line);
                        sutai.AppendChild(record);
                    }

                    newFile.Write( x.OuterXml );
                }

                newFile.Close();
            }
            catch (IOException e395)
            {
                Error.handler( e395.ToString() );
                line_count = ERROR_CODE;
            }
            catch (Exception e396)
            {
                Error.handler( e396.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int selecting_adding_ID()
        {
            byte few_fields;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            record = d.phraseElement(x, line, line_count);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            record = d.phraseElement(x, line, line_count);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        d = new SegmentedSummary(pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.segmented_summary(line);
                        record = d.phraseElement(x, line, line_count);
                        sutai.AppendChild(record);
                    }

                    newFile.Write( x.OuterXml );
                }

                newFile.Close();
            }
            catch (IOException e397)
            {
                Error.handler( e397.ToString() );
                line_count = ERROR_CODE;
            }
            catch (Exception e398)
            {
                Error.handler( e398.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int selecting_doubles_adding_ID()
        {
            byte few_fields;
            FieldsList f;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_double_from_field(line);
                            record = d.doubleElement(x, line, line_count);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_double_from_field(line);
                            record = d.doubleElement(x, line, line_count);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        f = new FieldsList();
                        d = new SegmentedSummary(f, pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.extracting_double_from_field(line);
                        record = d.doubleElement(x, line, line_count);
                        sutai.AppendChild(record);
                    }

                    newFile.Write( x.OuterXml );
                }

                newFile.Close();
            }
            catch (IOException e399)
            {
                Error.handler( e399.ToString() );
                line_count = ERROR_CODE;
            }
            catch (Exception e400)
            {
                Error.handler( e400.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int selecting_integers_adding_ID()
        {
            byte few_fields;
            FieldsList f;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_integer_from_field(line);
                            record = d.integerElement(x, line, line_count);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_integer_from_field(line);
                            record = d.integerElement(x, line, line_count);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        f = new FieldsList();
                        d = new SegmentedSummary(f, pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.extracting_integer_from_field(line);
                        record = d.integerElement(x, line, line_count);
                        sutai.AppendChild(record);
                    }

                    newFile.Write( x.OuterXml );
                }

                newFile.Close();
            }
            catch (IOException e401)
            {
                Error.handler( e401.ToString() );
                line_count = ERROR_CODE;
            }
            catch (Exception e402)
            {
                Error.handler( e402.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int skipping()
        {
            byte few_fields;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            record = d.phraseElement(x, line, GAP);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            record = d.phraseElement(x, line, GAP);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        d = new SegmentedSummary(pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.segmented_summary(line);
                        record = d.phraseElement(x, line, GAP);
                        sutai.AppendChild(record);
                    }

                    newFile.Write( x.OuterXml );
                }

                newFile.Close();
            }
            catch (IOException e445)
            {
                Error.handler( e445.ToString() );
                line_count = ERROR_CODE;
            }
            catch (Exception e446)
            {
                Error.handler( e446.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int skipping_adding_ID()
        {
            byte few_fields;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            record = d.phraseElement(x, line, line_count, GAP);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            record = d.phraseElement(x, line, line_count, GAP);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        d = new SegmentedSummary(pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.segmented_summary(line);
                        record = d.phraseElement(x, line, line_count, GAP);
                        sutai.AppendChild(record);
                    }

                    newFile.Write( x.OuterXml );
                }

                newFile.Close();
            }
            catch (IOException e447)
            {
                Error.handler( e447.ToString() );
                line_count = ERROR_CODE;
            }
            catch (Exception e448)
            {
                Error.handler( e448.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int skipping_showing_line_for_each_summary()
        {
            byte few_fields;
            HashSet<string> hashSet = new HashSet<string>();
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            d.compactCombo(GAP);
                            if (d.uniqueCombo(hashSet))
                            {
                                record = d.phraseElement(x, line);
                                sutai.AppendChild(record);
                            }

                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            d.compactCombo(GAP);
                            if ( d.uniqueCombo(hashSet) )
                            {
                                record = d.phraseElement(x, line);
                                sutai.AppendChild(record);
                            }

                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        d = new SegmentedSummary(pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.segmented_summary(line);
                        d.compactCombo(GAP);
                        if ( d.uniqueCombo(hashSet) )
                        {
                            record = d.phraseElement(x, line);
                            sutai.AppendChild(record);
                        }

                    }

                    newFile.Write( x.OuterXml );
                }

                newFile.Close();
            }
            catch (IOException e449)
            {
                Error.handler( e449.ToString() );
                line_count = ERROR_CODE;
            }
            catch (Exception e450)
            {
                Error.handler( e450.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int shift_jis_selecting(bool add_ID, bool descending, bool report_ID, bool unique_summary)
        {
            Block b;
            byte few_fields;
            CheckID c = new CheckID();
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            List<Block> blockArray = new List<Block>();
            List<string> items = new List<string>();
            RemoveControl d;
            string line, point, summary;
            string previous = EMPTY;

            try
            {
                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, pattern, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new RemoveControl(f, pattern, field_count, separator, remove_separators);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new Block(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, pattern, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new RemoveControl(f, pattern, field_count, separator, remove_separators);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new Block(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

            }
            catch (IOException e451)
            {
                Error.handler( e451.ToString() );
                exit_code = ERROR_CODE;
            }
            catch (Exception e452)
            {
                Error.handler( e452.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            if (descending)
            {
                for (int i = items.Count - 1; i >= 0; --i)
                {
                    Console.WriteLine(items[i]);
                }

            }
            else
            {
                items.ForEach(g => Console.WriteLine(g));
            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int shift_jis_selecting_except_dakuten(bool add_ID, bool descending, bool report_ID, bool substituted, bool unique_summary)
        {
            Block b;
            byte few_fields;
            CheckID c = new CheckID();
            char dakuten = (char)0xDE;
            char handakuten = (char)0xDF;
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            List<Block> blockArray = new List<Block>();
            List<string> items = new List<string>();
            ExceptDakuten d;
            string line, point, summary;
            string previous = EMPTY;

            if (substituted)
            {
                dakuten = 'H';
                handakuten = 'I';
            }

            try
            {
                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakuten(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new Block(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(line, summary );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(line, summary );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakuten(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new Block(line, summary );
                            blockArray.Add(b);
                        }

                    }

                }

            }
            catch (IOException e455)
            {
                Error.handler( e455.ToString() );
                exit_code = ERROR_CODE;
            }
            catch (Exception e456)
            {
                Error.handler( e456.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            if (descending)
            {
                for (int i = items.Count - 1; i >= 0; --i)
                {
                    Console.WriteLine(items[i]);
                }

            }
            else
            {
                items.ForEach(g => Console.WriteLine(g));
            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int shift_jis_selecting_for_case_insensitive_search(bool add_ID, bool descending, bool report_ID, bool unique_summary)
        {
            byte few_fields;
            CheckID c = new CheckID();
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            List<UpperCaseBlock> blockArray = new List<UpperCaseBlock>();
            List<string> items = new List<string>();
            RemoveControl d;
            string line, point, summary;
            string previous = EMPTY;
            UpperCaseBlock b;

            try
            {
                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, pattern, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new RemoveControl(f, pattern, field_count, separator, remove_separators);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, pattern, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new RemoveControl(f, pattern, field_count, separator, remove_separators);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new UpperCaseBlock(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

            }
            catch (IOException e459)
            {
                Error.handler( e459.ToString() );
                exit_code = ERROR_CODE;
            }
            catch (Exception e460)
            {
                Error.handler( e460.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            if (descending)
            {
                for (int i = items.Count - 1; i >= 0; --i)
                {
                    Console.WriteLine(items[i]);
                }

            }
            else
            {
                items.ForEach(g => Console.WriteLine(g));
            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int shift_jis_selecting_for_case_insensitive_search_except_dakuten(bool add_ID, bool descending, bool report_ID, bool substituted, bool unique_summary)
        {
            byte few_fields;
            CheckID c = new CheckID();
            char dakuten = (char)0xDE;
            char handakuten = (char)0xDF;
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            List<UpperCaseBlock> blockArray = new List<UpperCaseBlock>();
            List<string> items = new List<string>();
            ExceptDakuten d;
            string line, point, summary;
            string previous = EMPTY;
            UpperCaseBlock b;

            if (substituted)
            {
                dakuten = 'H';
                handakuten = 'I';
            }

            try
            {
                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakuten(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakuten(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new UpperCaseBlock(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

            }
            catch (IOException e463)
            {
                Error.handler( e463.ToString() );
                exit_code = ERROR_CODE;
            }
            catch (Exception e464)
            {
                Error.handler( e464.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            if (descending)
            {
                for (int i = items.Count - 1; i >= 0; --i)
                {
                    Console.WriteLine(items[i]);
                }

            }
            else
            {
                items.ForEach(g => Console.WriteLine(g));
            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int shift_jis_skipping(bool add_ID, bool descending, bool report_ID, bool unique_summary)
        {
            Block b;
            byte few_fields;
            CheckID c = new CheckID();
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            List<Block> blockArray = new List<Block>();
            List<string> items = new List<string>();
            RemoveControl d;
            string line, point, summary;
            string previous = EMPTY;

            try
            {
                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, pattern, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new RemoveControl(f, pattern, field_count, separator, remove_separators);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new Block(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, pattern, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new RemoveControl(f, pattern, field_count, separator, remove_separators);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new Block(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

            }
            catch (IOException e467)
            {
                Error.handler( e467.ToString() );
                exit_code = ERROR_CODE;
            }
            catch (Exception e466)
            {
                Error.handler( e466.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            if (descending)
            {
                for (int i = items.Count - 1; i >= 0; --i)
                {
                    Console.WriteLine(items[i]);
                }

            }
            else
            {
                items.ForEach(g => Console.WriteLine(g));
            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int shift_jis_skipping_except_dakuten(bool add_ID, bool descending, bool report_ID, bool substituted, bool unique_summary)
        {
            Block b;
            byte few_fields;
            CheckID c = new CheckID();
            char dakuten = (char)0xDE;
            char handakuten = (char)0xDF;
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            List<Block> blockArray = new List<Block>();
            List<string> items = new List<string>();
            ExceptDakuten d;
            string line, point, summary;
            string previous = EMPTY;

            if (substituted)
            {
                dakuten = 'H';
                handakuten = 'I';
            }

            try
            {
                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakuten(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new Block(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakuten(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new Block(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

            }
            catch (IOException e469)
            {
                Error.handler( e469.ToString() );
                exit_code = ERROR_CODE;
            }
            catch (Exception e470)
            {
                Error.handler( e470.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            if (descending)
            {
                for (int i = items.Count - 1; i >= 0; --i)
                {
                    Console.WriteLine(items[i]);
                }

            }
            else
            {
                items.ForEach(g => Console.WriteLine(g));
            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int shift_jis_skipping_for_case_insensitive_search(bool add_ID, bool descending, bool report_ID, bool unique_summary)
        {
            byte few_fields;
            CheckID c = new CheckID();
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            List<UpperCaseBlock> blockArray = new List<UpperCaseBlock>();
            List<string> items = new List<string>();
            RemoveControl d;
            string line, point, summary;
            string previous = EMPTY;
            UpperCaseBlock b;

            try
            {
                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, pattern, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new RemoveControl(f, pattern, field_count, separator, remove_separators);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, pattern, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new RemoveControl(f, pattern, field_count, separator, remove_separators);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new UpperCaseBlock(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

            }
            catch (IOException e473)
            {
                Error.handler( e473.ToString() );
                exit_code = ERROR_CODE;
            }
            catch (Exception e483)
            {
                Error.handler( e483.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            if (descending)
            {
                for (int i = items.Count - 1; i >= 0; --i)
                {
                    Console.WriteLine(items[i]);
                }

            }
            else
            {
                items.ForEach(g => Console.WriteLine(g));
            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int shift_jis_skipping_for_case_insensitive_search_except_dakuten(bool add_ID, bool descending, bool report_ID, bool substituted, bool unique_summary)
        {
            byte few_fields;
            CheckID c = new CheckID();
            char dakuten = (char)0xDE;
            char handakuten = (char)0xDF;
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            List<UpperCaseBlock> blockArray = new List<UpperCaseBlock>();
            List<string> items = new List<string>();
            ExceptDakuten d;
            string line, point, summary;
            string previous = EMPTY;
            UpperCaseBlock b;

            if (substituted)
            {
                dakuten = 'H';
                handakuten = 'I';
            }

            try
            {
                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakuten(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakuten(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new UpperCaseBlock(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

            }
            catch (IOException e472)
            {
                Error.handler( e472.ToString() );
                exit_code = ERROR_CODE;
            }
            catch (Exception e468)
            {
                Error.handler( e468.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            if (descending)
            {
                for (int i = items.Count - 1; i >= 0; --i)
                {
                    Console.WriteLine(items[i]);
                }

            }
            else
            {
                items.ForEach(g => Console.WriteLine(g));
            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int euc_jp_selecting_except_dakuten(bool add_ID, bool descending, bool report_ID, bool substituted, bool unique_summary)
        {
            Block b;
            byte few_fields;
            CheckID c = new CheckID();
            char dakuten = (char)0xDE;
            char handakuten = (char)0xDF;
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            List<Block> blockArray = new List<Block>();
            List<string> items = new List<string>();
            ExceptDakutenEUC d;
            string line, point, summary;
            string previous = EMPTY;

            if (substituted)
            {
                dakuten = 'H';
                handakuten = 'I';
            }

            try
            {
                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakutenEUC(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new Block(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(line, summary );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(line, summary );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakutenEUC(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new Block(line, summary );
                            blockArray.Add(b);
                        }

                    }

                }

            }
            catch (IOException e526)
            {
                Error.handler( e526.ToString() );
                exit_code = ERROR_CODE;
            }
            catch (Exception e527)
            {
                Error.handler( e527.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            if (descending)
            {
                for (int i = items.Count - 1; i >= 0; --i)
                {
                    Console.WriteLine(items[i]);
                }

            }
            else
            {
                items.ForEach(g => Console.WriteLine(g));
            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int euc_jp_selecting_for_case_insensitive_search_except_dakuten(bool add_ID, bool descending, bool report_ID, bool substituted, bool unique_summary)
        {
            byte few_fields;
            CheckID c = new CheckID();
            char dakuten = (char)0xDE;
            char handakuten = (char)0xDF;
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            List<UpperCaseBlock> blockArray = new List<UpperCaseBlock>();
            List<string> items = new List<string>();
            ExceptDakutenEUC d;
            string line, point, summary;
            string previous = EMPTY;
            UpperCaseBlock b;

            if (substituted)
            {
                dakuten = 'H';
                handakuten = 'I';
            }

            try
            {
                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakutenEUC(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakutenEUC(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new UpperCaseBlock(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

            }
            catch (IOException e528)
            {
                Error.handler( e528.ToString() );
                exit_code = ERROR_CODE;
            }
            catch (Exception e529)
            {
                Error.handler( e529.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            if (descending)
            {
                for (int i = items.Count - 1; i >= 0; --i)
                {
                    Console.WriteLine(items[i]);
                }

            }
            else
            {
                items.ForEach(g => Console.WriteLine(g));
            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int euc_jp_skipping_except_dakuten(bool add_ID, bool descending, bool report_ID, bool substituted, bool unique_summary)
        {
            Block b;
            byte few_fields;
            CheckID c = new CheckID();
            char dakuten = (char)0xDE;
            char handakuten = (char)0xDF;
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            List<Block> blockArray = new List<Block>();
            List<string> items = new List<string>();
            ExceptDakutenEUC d;
            string line, point, summary;
            string previous = EMPTY;

            if (substituted)
            {
                dakuten = 'H';
                handakuten = 'I';
            }

            try
            {
                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakutenEUC(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new Block(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakutenEUC(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new Block(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

            }
            catch (IOException e530)
            {
                Error.handler( e530.ToString() );
                exit_code = ERROR_CODE;
            }
            catch (Exception e531)
            {
                Error.handler( e531.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            if (descending)
            {
                for (int i = items.Count - 1; i >= 0; --i)
                {
                    Console.WriteLine(items[i]);
                }

            }
            else
            {
                items.ForEach(g => Console.WriteLine(g));
            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int euc_jp_skipping_for_case_insensitive_search_except_dakuten(bool add_ID, bool descending, bool report_ID, bool substituted, bool unique_summary)
        {
            byte few_fields;
            CheckID c = new CheckID();
            char dakuten = (char)0xDE;
            char handakuten = (char)0xDF;
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            List<UpperCaseBlock> blockArray = new List<UpperCaseBlock>();
            List<string> items = new List<string>();
            ExceptDakutenEUC d;
            string line, point, summary;
            string previous = EMPTY;
            UpperCaseBlock b;

            if (substituted)
            {
                dakuten = 'H';
                handakuten = 'I';
            }

            try
            {
                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakutenEUC(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakutenEUC(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new UpperCaseBlock(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

            }
            catch (IOException e532)
            {
                Error.handler( e532.ToString() );
                exit_code = ERROR_CODE;
            }
            catch (Exception e533)
            {
                Error.handler( e533.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            if (descending)
            {
                for (int i = items.Count - 1; i >= 0; --i)
                {
                    Console.WriteLine(items[i]);
                }

            }
            else
            {
                items.ForEach(g => Console.WriteLine(g));
            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

    }
}
